LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.all;
USE IEEE.STD_LOGIC_ARITH.all;
USE IEEE.STD_LOGIC_UNSIGNED.all;

ENTITY ludion_2d IS
   PORT( rouge, vert, bleu 				: OUT std_logic;
         sync_verti						: IN std_logic;
         pixel_colone, pixel_ligne		: IN std_logic_vector (10 downto 0));	
END ludion_2d;


architecture archi of ludion_2d is

--ce programme fait rebondir une "balle carre" 
--sur les bords de l'cran
--le dplacement du carr sera recalcul  chaque top synchro verticale
--le fond de l'cran est rouge, la "balle" est blanche

-- taille reprsente (en pixel) un demi ct du carr 
CONSTANT taille		:INTEGER :=10;

--balle est au NL1 lorsque le carr se trouve sur les pixels rafrachi
--ball_Y_pos et ball_X_pos donne la position du centre du carr
--mouv_Y et mouv_X donne le dplacement du carr
--mouv_Y et mouv_X pouvant tre ngatifs, le plus simple est de prendre des entiers

SIGNAL balle				: std_logic;
SIGNAL ball_Y_pos, ball_X_pos	: std_logic_vector(10 DOWNTO 0);
SIGNAL mouv_Y			: INTEGER RANGE -7 TO 7 :=2;
SIGNAL mouv_X			: INTEGER RANGE -7 TO 7 :=2;


BEGIN

-- le fond d'cran est rouge
rouge <=  '1';
-- le carr est blanc, la position du carr correspondant aux pixels o balle =1
vert <=  balle;
bleu <=  balle;

--en fonction du pixel rafrachi, on dtermine la couleur
balle <= '1' when 		(Ball_X_pos - taille <= pixel_colone )
				AND 	(Ball_X_pos + taille >= pixel_colone) 
				AND 	(Ball_Y_pos - taille <= pixel_ligne ) 
				AND	(Ball_Y_pos + taille >= pixel_ligne )
		
				else	'0';


--calcul de la position de la "balle" sur l'axe Y
--qui se dplace de +/-2 pixels  chaque top synchro verticale
mouvement_balle_Y: process
	BEGIN
		WAIT UNTIL sync_verti'event and sync_verti = '1';
--sens du dplacement suivant que nous sommmes en haut ou en bas de l'cran
		IF ball_Y_pos >= 599 - taille THEN	mouv_Y <=-2;		
		ELSIF ball_Y_pos <= taille THEN	mouv_Y <=2;
		END IF;

-- Calcul de la prochaine position de Y
		ball_Y_pos <= ball_Y_pos + mouv_Y;
END process mouvement_balle_Y;


--calcul de la position de la "balle" sur l'axe X
--qui se dplace de +/-2 pixels  chaque top synchro verticale
mouvement_balle_X: process
	BEGIN
		WAIT UNTIL sync_verti'event and sync_verti = '1';
--sens du dplacement suivant que nous sommmes  gauche ou  droite de l'cran
			IF ball_X_pos >= 799 - taille THEN	mouv_X <=-2;		
			ELSIF ball_X_pos <= taille THEN	mouv_X <=2;
			END IF;

-- Calcul de la prochaine position de X
		ball_X_pos <= ball_X_pos + mouv_X;
END process mouvement_balle_X;

END archi;
