LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.all;
USE IEEE.STD_LOGIC_ARITH.all;
USE IEEE.STD_LOGIC_UNSIGNED.all;

ENTITY ludion_3d IS
PORT( rouge, vert, bleu 		: OUT std_logic;
		sync_verti			: IN std_logic;
		pixel_colone, pixel_ligne		: IN std_logic_vector (10 downto 0));	
END ludion_3d;

architecture archi of ludion_3d is

--ce programme fait rebomdir une "balle carre" en 3d
--le dplacement du carr sera recalcul  chaque top synchro verticale
--le fond de l'cran est rouge, la "balle" est blanche

--balle est au NL1 lorsque le carr se trouve sur les pixels rafrachi
--ball_Y_pos et ball_X_pos donne la position du centre du carr
--mouv_Y et mouv_X donne le dplacement du carr
--mouv_Y et mouv_X pouvant tre ngatifs, le plus simple est de prendre des entiers
--taille reprsente (en pixel) un demi ct du carr 
--pour diminuer la vitesse de dplacement sur l'axe Z, on divise sync_verti par 4
--DIV_Z est la sortie du compteur et HOR_Z le signal divis par 4
SIGNAL balle				: std_logic;
SIGNAL ball_Y_pos, ball_X_pos	: std_logic_vector(10 DOWNTO 0);
SIGNAL mouv_Y			: INTEGER RANGE -7 TO 7 :=2;
SIGNAL mouv_X			: INTEGER RANGE -7 TO 7 :=2;
SIGNAL mouv_Z			: INTEGER RANGE -7 TO 7 :=1;
SIGNAL taille				: INTEGER RANGE 0 TO 63 :=10;
SIGNAL DIV_Z				: std_logic_vector(1 DOWNTO 0);
SIGNAL HOR_Z			: std_logic;

BEGIN

-- le fond d'cran est bleu
bleu <=  '1';
-- le carr est blanc, la position du carr correspondant aux pixels o balle =1
vert <=  balle;
rouge <=  balle;

--en fonction du pixel rafrachi, on dtermine la couleur
balle <= '1' when 	(Ball_X_pos - taille <= pixel_colone )
			AND 	(Ball_X_pos + taille >= pixel_colone) 
			AND 	(Ball_Y_pos - taille <= pixel_ligne ) 
			AND		(Ball_Y_pos + taille >= pixel_ligne )
		else		'0';


--calcul de la position de la "balle" sur l'axe Y
--qui se dplace de +/-2 pixels  chaque top synchro verticale
mouvement_balle_Y: process
	BEGIN
	WAIT UNTIL sync_verti'event and sync_verti = '1';

--sens du dplacement suivant que nous sommmes en haut ou en bas de l'cran
	IF ball_Y_pos >= 599 - taille THEN	mouv_Y <=-2;		
	ELSIF ball_Y_pos <= taille THEN	mouv_Y <=2;
	END IF;


-- Calcul de la prochaine position de Y
	ball_Y_pos <= ball_Y_pos + mouv_Y;

END process mouvement_balle_Y;

--calcul de la position de la "balle" sur l'axe X
--qui se dplace de +/-2 pixels  chaque top synchro verticale
mouvement_balle_X: process
	BEGIN
	WAIT UNTIL sync_verti'event and sync_verti = '1';

--sens du dplacement suivant que nous sommmes  droite ou  gauche de l'cran
		IF ball_X_pos >= 799 - taille THEN	mouv_X <=-2;		
		ELSIF ball_X_pos <= taille THEN	mouv_X <=2;
		END IF;

-- Calcul de la prochaine position de X
		ball_X_pos <= ball_X_pos + mouv_X;

END process mouvement_balle_X;

--obtention de HOR_Z  partir de sync_verti
horloge_axe_Z: process
		BEGIN
		WAIT UNTIL sync_verti'event and sync_verti = '1';						
		DIV_Z <= DIV_Z +1;
			if DIV_Z="11" then HOR_Z <='1';
			else HOR_Z <='0';
			end if;
END process horloge_axe_Z;

--calcul de l'illusion de dplacement suivant l'axe z
mouvement_balle_Z: process
		BEGIN
		WAIT UNTIL HOR_Z'event and HOR_Z = '1';
			IF taille >= 29 THEN		mouv_Z <= -1;		
			ELSIF taille <= 9 THEN		mouv_Z <= 1;
			END IF;

		taille <= taille + mouv_Z;

END process mouvement_balle_Z;


END archi;
